package csci1902.hw3;

import junit.framework.TestCase;
import java.awt.Point;

/**
 * Test case to test the Entity class.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 */
public class EntityTest extends TestCase {
    private World testWorld;
    private TestEntity entity;
    
    protected void setUp() {
        testWorld = new World(10, 10);
        entity = new TestEntity(new Point(2,3));
    }
    
    /**
     * Test that the initial state is correct.
     */
    public void testInitialState() {
        assertEquals(new Point(2, 3), entity.getPosition());
        assertSame(testWorld, entity.getWorld());
    }
    
    /**
     * Test that setting the next position is acceptable, and that committing
     * applies the change.
     */
    public void testPositionStep() {
        entity.setNextPos(new Point(5,7));
        assertEquals(new Point(2, 3), entity.getPosition());
        entity.update();
        assertEquals(new Point(5, 7), entity.getPosition());
    }
    
    /**
     * Test that an entity is not a food source.
     */
    public void testFoodSource() {
        assertFalse(entity.isFoodSource());
        try {
            entity.collectFood(10);
            fail("collectFood didn't throw");
        } catch (UnsupportedOperationException e) {
            // no-op, expected
        }
    }
    
    /**
     * Test that an entity is not a food sink.
     */
    public void testFoodSink() {
        assertFalse(entity.isFoodSink());
        try {
            entity.depositFood(10);
            fail("depositFood didn't throw");
        } catch (UnsupportedOperationException e) {
            // no-op, expected
        }
    }
    
    /**
     * Test than an entity is not edible.
     */
    public void testEdible() {
        assertFalse(entity.isEdible());
        try {
            entity.eat();
            fail("eat didn't throw");
        } catch (UnsupportedOperationException e) {
            // no-op, expected
        }
    }
    
    public void testWasteSource() {
        assertFalse(entity.isWasteSource());
        try {
            entity.collectWaste(3);
            fail("collectWaste didn't throw");
        } catch (UnsupportedOperationException e) {
            // no-op, expected
        }
    }
    
    public void testWasteSink() {
        assertFalse(entity.isWasteSink());
        try {
            entity.depositWaste(3);
            fail("depositWaste didn't throw");
        } catch (UnsupportedOperationException e) {
            // no-op, expected
        }
    }
    
    public void testDie() {
        entity.die();
        assertTrue(entity.isDying());
        assertTrue(entity.isAlive());
        entity.update();
        assertFalse(entity.isAlive());
    }
    
    /**
     * A subclass of Entity for testing correctness.
     * @author Michael Ekstrand <ekstrand@cs.umn.edu>
     */
    private class TestEntity extends Entity {
        public TestEntity(Point p) {
            super(testWorld, p);
        }
        public void setNextPos(Point p) {
            super.setNextPosition(p);
        }
        public void die() {
            super.die();
        }
    }
}
