package csci1902.hw3;

import junit.framework.TestCase;
import java.awt.Point;
import java.util.List;
import java.util.ArrayList;

public class SpiderTest extends TestCase {
    private World world;
    private Spider spider;
    private FakeRandomGenerator rng;
    private List<Entity> neighbors;
    
    protected void setUp() {
        rng = new FakeRandomGenerator();
        world = new World(10, 10);
        world.setRandomGenerator(rng);
        spider = new Spider(world, new Point(5,5));
        neighbors = new ArrayList<Entity>();
    }
    
    /**
     * Test that the spider moves randomly.
     */
    public void testMoveRandomly() {
        world.setRandomGenerator(rng);
        rng.addNumber(0.05);
        spider.act(neighbors);
        assertEquals(new Point(5,5), spider.getPosition());
        spider.update();
        assertEquals(new Point(4,4), spider.getPosition());
        
        rng.addNumber(0.6);
        spider.act(neighbors);
        spider.update();
        assertEquals(new Point(5,4), spider.getPosition());
    }
    
    /**
     * Test that a spider can try to eat an ant.
     */
    public void testEatAnt() {
        Entity e = new ForagerAnt(100, world, new Point(4,5));
        rng.addNumber(0.4);
        neighbors.add(e);
        
        spider.act(neighbors);
        assertTrue(e.isDying());
    }
    
    /**
     * Test that a spider dies when it goes 1000 times without eating an ant.
     */
    public void testDie() {
        for (int i = 1; i < 1000; i++) {
            spider.act(neighbors);
            assertFalse(spider.isDying());
            spider.update();
            assertTrue(spider.isAlive());
        }
        
        spider.act(neighbors);
        assertTrue(spider.isDying());
        spider.update();
        assertFalse(spider.isAlive());
    }
    
    /**
     * Test that succesfully eating an ant prolongs a spider's life.
     */
    public void testGainEnergy() {
        RandomGenerator g = new RealRandomGenerator();
        world.setRandomGenerator(g);
        for (int i = 0; i < 500; i++) {
            spider.act(neighbors);
            spider.update();
        }
        Point p = spider.getPosition();
        Point ap = new Point(p.x+1, p.y); 
        neighbors.add(new ForagerAnt(100, world, ap));
        rng.addNumber(0.3);
        world.setRandomGenerator(rng);
        spider.act(neighbors);
        spider.update();
        neighbors.clear();
        world.setRandomGenerator(g);
        for (int i = 1; i < 510; i++) {
            spider.act(neighbors);
            spider.update();
            assertTrue(spider.isAlive());
        }
        spider.act(neighbors);
        spider.update();
        assertFalse(spider.isAlive());
    }
}
